<?php

/**
 * Debug Bar loader template.
 *
 * It uses Font Awesome by Dave Gandy - http://fontawesome.io
 *
 * This file is part of the Tracy (https://tracy.nette.org)
 * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
 */

namespace Tracy;

use Tracy;

?>




<!-- Tracy Debug Bar -->
<div id="tracy-debug" data-id="<?= htmlspecialchars($contentId) ?>"></div>
<link rel="stylesheet" href="<?= htmlspecialchars("$path?"), http_build_query(['_tracy_bar' => 'css', 'v' => Debugger::VERSION] + $stopXdebug, NULL, '&amp;') ?>" id="tracy-debug-style" class="tracy-debug" />
<script src="<?= htmlspecialchars("$path?"), http_build_query(['_tracy_bar' => 'js', 'v' => Debugger::VERSION] + $stopXdebug, NULL, '&amp;') ?>" id="tracy-debug-script"></script>
<?php if ($contentId): ?>
<script src="<?= htmlspecialchars("$path?"), http_build_query(['_tracy_bar' => "content.$contentId"] + $stopXdebug, NULL, '&amp;') ?>"></script>
<?php else: ?>
<script>
Tracy.Debug.init(<?= json_encode($content) ?>, <?= json_encode($dumps) ?>);
</script>
<?php endif ?>
