html {
	font: 15px/1.5 Tahoma, sans-serif;
	color: #333;
	background: #fff;
	margin: 0 1.6em;
	padding: 0;
}

html.arrow {
	background: #fff url(arrow.png) no-repeat bottom right;
	height: 100%;
}

h1 {
	font-size: 210%;
	font-weight: normal;
	color: #036;
}

pre.tracy-dump {
	border: 1px solid silver;
	padding: 1em;
	margin: 1em 0;
}

a {
	color: #000080;
}
