<?php

/**
 * This file is part of the Nette Framework (https://nette.org)
 * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
 */

namespace Nette;


/**
 * Framework version.
 */
class Framework
{

	const NAME = 'Nette Framework',
		VERSION = '2.4',
		VERSION_ID = 20400,
		REVISION = '';

}
