<?php

/**
 * This file is part of the Nette Framework (https://nette.org)
 * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
 */

namespace Nette\Utils;


interface IHtmlString
{

	/**
	 * @return string in HTML format
	 */
	function __toString();

}
