(function(){Tracy=window.Tracy||{};var f=Tracy.BlueScreen={},g;f.init=function(b){for(var d=document.getElementById("tracy-bs"),a=0,e=[];a<document.styleSheets.length;a++){var c=document.styleSheets[a];c.ownerNode.classList.contains("tracy-debug")||(c.oldDisabled=c.disabled,c.disabled=!0,e.push(c))}document.getElementById("tracy-bs-toggle").addEventListener("tracy-toggle",function(h){h=this.classList.contains("tracy-collapsed");for(a=0;a<e.length;a++)e[a].disabled=h?e[a].oldDisabled:!0});b||(document.body.appendChild(d),
b=location.href+document.getElementById("tracy-bs-error").textContent,Tracy.Toggle.persist(d,sessionStorage.getItem("tracy-toggles-bskey")===b),sessionStorage.setItem("tracy-toggles-bskey",b));g||(g=!0,document.addEventListener("keyup",function(a){27!==a.keyCode||(a.shiftKey||a.altKey||a.ctrlKey||a.metaKey)||Tracy.Toggle.toggle(document.getElementById("tracy-bs-toggle"))}))};f.loadAjax=function(b,d){var a=document.getElementById("tracy-bs");a&&a.parentNode.removeChild(a);document.body.insertAdjacentHTML("beforeend",
b);a=document.getElementById("tracy-bs");Tracy.Dumper.init(d,a);f.init(!0);window.scrollTo(0,0)}})();