body {
	font: 16px/1.5 "Trebuchet MS", "Geneva CE", lucida, sans-serif;
	color: #333;
	background-color: #fff;
	margin: 2em;
}

h1 {
	font-size: 1.9em;
	color: #3484D2;
}

h2 {
	font-size: 1.2em;
	color: #3484D2;
}

#content {
	width: 770px;
	margin: 0 5px;
}

a {
	color: #000080;
}

#logged-in {
	margin-top: 3em;
	font-size: 90%;
}

div.flash {
	color: black;
	background: #FFFFDD;
	border: 1px solid #FFD700;
	padding: 1em;
	margin: 1em 0;
}



/*------------------------------------------------------------------*/


table.grid {
	padding: 0;
	margin: 0;
	border-collapse:collapse;
}

table.grid td, table.grid th {
	background: #fff;
	border: 1px solid #add4fb;
	padding: 6px 6px 6px 12px;
}

table.grid th {
	color: #7a7772;
	background: #E4F1FC;
	text-align: left;
	font-weight: normal;
	font-size: 80%;
}

table.grid .alt td {
	background: #f8f8f0;
}



/*------------------------------------------------------------------*/



form {
	max-width: 500px;
	padding: .8em 1.6em;
	background: #E4F1FC;
	border: solid 2px #add4fb;
}

form input {
	margin: 2px 0;
	font-size: 100%;
}

form input.default {
	font-weight: bold;
	font-size: 105%;
}

form input.text {
	padding: 4px 2px;
	border: solid 1px #add4fb;
	min-width: 200px;
}

form label {
	width: 100px;
	display: block;
	text-align: right;
	margin-right: 5px;
	font-weight: normal;
}

form .required label {
	font-weight: bold;
}
