html {
	font: 16px/1.5 "Trebuchet MS", "Geneva CE", lucida, sans-serif;
	border-top: 4.7em solid #F4EBDB;
}

body {
	max-width: 990px;
	margin: -4.7em auto 0;
	background: white;
	color: #333;
}

h1 {
	font: 1.9em/1.5 sans-serif;
	margin: .5em 0 1.5em;
	background: url(https://files.nette.org/icons/logo-e1.png) right center no-repeat;
	color: #7A7772;
	text-shadow: 1px 1px 0 white;
}

h2 {
	font-size: 120%;
	color: #3484D2;
}

#content {
	width: 770px;
	margin: 0 5px;
	border: 1px dotted gray;
}


#path {
	margin: 50px 0;
}

#module {
	background: #fee7bc;
	padding: 30px 10px;
	=padding: 10px;
	font-weight: bold;
}

#presenter {
	background: #afcbe3;
	padding: 20px 10px 20px 0;
	=padding: 10px 10px 10px 0;
}

#view {
	background: #c4e8c4;
	padding: 10px 10px 10px 0;
	font-weight: normal;
}

fieldset {
	margin: 1em;
	padding: 1em;
}
