<?php

/**
 * Default error page.
 * @param  bool $logged
 */

namespace Tracy;

?>
<!DOCTYPE html><!-- "' --></script></style></noscript></xmp>
<meta charset="utf-8">
<meta name=robots content=noindex>
<meta name=generator content="Tracy">
<title>Server Error</title>

<style>#tracy-error{background:white;width:500px;margin:70px auto;padding:10px 20px}#tracy-error h1{font:bold 47px/1.5 sans-serif;background:none;color:#333;margin:.6em 0}#tracy-error p{font:21px/1.5 Georgia,serif;background:none;color:#333;margin:1.5em 0}#tracy-error small{font-size:70%;color:gray}</style>

<div id=tracy-error>
	<h1>Server Error</h1>

	<p>We're sorry! The server encountered an internal error and
	was unable to complete your request. Please try again later.</p>

	<p><small>error 500<?php if (!$logged): ?><br>Tracy is unable to log error.<?php endif ?></small></p>
</div>
