html {
	font: 16px/1.5 sans-serif;
	border-top: 4.7em solid #F4EBDB;
}

body {
	max-width: 990px;
	margin: -4.7em auto 0;
	background: white;
	color: #333;
}

h1 {
	font-size: 1.9em;
	margin: .5em 0 1.5em;
	background: url(https://files.nette.org/icons/logo-e1.png) right center no-repeat;
	color: #7A7772;
	text-shadow: 1px 1px 0 white;
}

img {
	border: none;
}

.fifteen img {
	display: block;
	border-right: 1px solid white;
	border-bottom: 1px solid white;
	position: relative;
}

.fifteen {
	border-collapse: collapse;
}

.fifteen td {
	padding: 0;
	background: url('../images/game0.jpg') 0 0 no-repeat;
	position: relative;
	=position: inherit;
}

#ajax-spinner {
	margin: 15px 0 0 15px;
	padding: 13px;
	background: white url('../images/spinner.gif') no-repeat 50% 50%;
	font-size: 0;
	z-index: 123456;
}
